#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "bergenListener.h"
#include "bergenServer.h"
#include "bergenRATSource.h"


// The constructor must be given the name of the RAT source which the sound
// will play, in addition to the server object pointer. The Sound Object
// itself is created by snerd on snerd's side.Snerd is managing the data and
// return just the Handle to it
bergenRATSource::bergenRATSource(char *source,bergenServer *server) : bergenSound(server)
	{
	source_ = strdup(source);
	createRemote();
	}


// Sends message to the associated server to create a sound sample 
// and waits for the resulting handle to store
void bergenRATSource::createRemote(void)
	{
	if (server_)
		{
		char cmd[256];
		sprintf(cmd,"-1 new ratsource %s %d",source_,server_->listener()->handle());
		server_->sendMessage(cmd);
		handle_ = server_->receiveHandle();
		}
	else
		handle_ = 0;
	}


// Sets the RAT source identifier 
void bergenRATSource::setSource(char* source)
	{
	if (source)
		{
		source_ = strdup(source);
		if (server_)
			{
			char message[256];
			sprintf(message,"%d source %s", handle_, source_);
			server_->sendMessage(message);
			}
		}
	}

// Sets the gain of the RAT source
void bergenRATSource::setSourceGain(float val)
	{
	if (server_)
		{
		char message[256];
		sprintf(message,"%d gain %f", handle_, val);
		server_->sendMessage(message);
		}
	}


// Enables 3D rendering for the RAT application
void bergenRATSource::set3DEnable(int val)
	{
	if (server_)
		{
		char message[256];
		sprintf(message,"%d enable %d", handle_, val);
		server_->sendMessage(message);
		}
	}


// Sets the azimuth of the RAT source for 3D postional sound
void bergenRATSource::setSource3DAzimuth(float val)
	{
	if (server_)
		{
		char message[256];
		sprintf(message,"%d azimuth %f", handle_, val);
		server_->sendMessage(message);
		}
	}
