#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include "bergenUDPSocket.h"


// Creates socket for communication and binds to the given port 
bergenUDPSocket::bergenUDPSocket(int port)
	{
	struct sockaddr_in addr;
	socket_ = socket(AF_INET, SOCK_DGRAM, 0);
	if (socket_ == -1)
		{
		fprintf(stderr,"ERROR: bergenUDPSocket failed to create a socket\n");
		perror("socket");
		return;
		}
	memset(&addr,0,sizeof(addr));
	addr.sin_addr.s_addr = INADDR_ANY;
	addr.sin_family = AF_INET;
	addr.sin_port = port;
	if (bind(socket_, (struct sockaddr *)&addr, sizeof(addr)) < 0)
		{
		fprintf(stderr,"ERROR: bergenUDPSocket failed to bind to port %d\n",
			port);
		perror("bind");
		socket_ = -1;
		return;
		}
	}


// Sets or resets blocking state
void bergenUDPSocket::setBlocking(int block)
	{
	if (socket_ == -1)
		return;
	if (block)
		fcntl(socket_, F_SETFL, fcntl(socket_, F_GETFL) & (~FNDELAY));
	else
		fcntl(socket_, F_SETFL, fcntl(socket_, F_GETFL) | FNDELAY);
	}


// Tries to find host and if successfull sets address and port for future 
void bergenUDPSocket::setDestination(char *host,int port)
	{
	struct hostent *hp;
	if (!(hp = gethostbyname(host)))
		{
		fprintf(stderr,"ERROR: bergenUDPSocket::setDestination failed "
			"to get address for \"%s\"\n",host);
		perror("gethostbyname");
		return;
		}
	int curPort = destAddr_.sin_port;
	memset(&destAddr_,0,sizeof(destAddr_));
	memcpy(&destAddr_.sin_addr, hp->h_addr, hp->h_length);
	destAddr_.sin_family = hp->h_addrtype;
	if (port >= 0)
		destAddr_.sin_port = port;
	else
		destAddr_.sin_port = curPort;
	}


// sends a buffer using opened socket
void bergenUDPSocket::send(void *buffer,int size)
	{
	if (socket_ == -1)
		return;
	sendto(socket_,buffer,size,0,(struct sockaddr *)&destAddr_,sizeof(destAddr_));
	}


// Receives an incomming message 
int bergenUDPSocket::receive(void *buffer,int size)
	{
	if (socket_ == -1)
		return 0;
	struct sockaddr_in fromAddr;
#ifdef __linux__
	unsigned int fromLen;
#else
	int fromLen;
#endif
	fromLen = sizeof(fromAddr);
	int retval = recvfrom(socket_,buffer,size,0,(struct sockaddr *)&fromAddr,&fromLen);
	lastFromAddr_ = fromAddr;
	return retval;
	}


// sends a buffer back to sender
void bergenUDPSocket::reply(void *buffer,int size)
	{
	if (socket_ == -1)
		return;
	sendto(socket_,buffer,size,0,(struct sockaddr *)&lastFromAddr_,sizeof(lastFromAddr_));
	}


char* bergenUDPSocket::lastFromAddress(void)
	{
	int ip = lastFromAddr_.sin_addr.s_addr;
#ifdef __sgi
	sprintf(lastFrom_ ,"%d.%d.%d.%d", (int)(ip>>24)&0xff,
		(int)(ip>>16)&0xff,
		(int)(ip>>8)&0xff,
		(int)ip&0xff);
#else
	sprintf(lastFrom_ ,"%d.%d.%d.%d", (int)ip&0xff,
		(int)(ip>>8)&0xff,
		(int)(ip>>16)&0xff,
		(int)(ip>>24)&0xff);
#endif
	return lastFrom_;
	}
