/******************************************************************************
 * Bergen Sound Server Spatialized Edition                                    *
 * SpatializedBergenTest Demo Application Sourcefile                          *
 *                                                                            *
 * Copyright 2002 Severin S. Todt - University of Applied Sciences Wedel      *
 *                s.todt@fh-wedel.de                                          *
 *                                                                            *
 * The Bergen Sound Server in current release is based on the                 *
 * Bergen Sound Server, edited 2000 by Dave Pape                              *
 *                                                                            *
 * The Bergen Sound Server is developed within a project at the               *
 * Electronic Visualization Laboratory at the University of Illinois at       *
 * Chicago (www.evl.uic.edu) and is part of the master thesis written by      *
 * Severin S. Todt at the University of Applied Sciences - Wedel.             *
 * (www.fh-wedel.de)                                                          *
 *                                                                            *
 ******************************************************************************
 * Modification history :                                                     *
 * - initial setup                                                            *
 *                                                                            *
 *                                                   last modified : 09/17/02 *
 ******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <math.h>
#include "bergenServer.h"
#include "bergenListener.h"
#include "bergenTone.h"
#include "bergenWhiteNoise.h"
#include "bergenSample.h"


enum SOUNDTYPE {TONE,WHITENOISE,SAMPLE};
enum MOVESCHEME {NO_MOVESCHEME,DISTANCE_RIGHT,DISTANCE_FRONT,DISTANCE_LEFT,DISTANCE_BACK,CIRCLE_HORIZONTAL,CIRCLE_VERTICAL,SPIRAL};

void help()
	{
	printf("********************************************************\n");
	printf(" Bergen Sound Server Spatialized Sound Extension Demo\n");
	printf(" written 2002 by Severin S. Todt - to@fh-wedel.de\n");
	printf("\n");
	/*create sound */
	printf("- - - - - - - - - - - - - - - - - - - - - - - - - - - - \n");
	printf(" Usage :\n");
	printf(" spatializedBergenTest [-option value]*\n");
	printf(" Options are :\n");
	printf(" -moveScheme \n");
	printf("    with values\n");
	printf("    0 : NO_MOVESCHEME\n");
	printf("    1 : DISTANCE_RIGHT\n");
	printf("    2 : DISTANCE_FRONT\n");
	printf("    3 : DISTANCE_LEFT\n");
	printf("    4 : DISTANCE_BACK\n");
	printf("    5 : CIRCLE_HORIZONTAL\n");
	printf("    6 : CIRCLE_VERTICAL\n");
	printf("    7 : SPIRAL\n");
	printf(" -soundType \n");
	printf("    with values\n");
	printf("    0 : TONE\n");
	printf("    1 : WHITENOISE\n");
	printf("    2 : SAMPLE\n");
	printf(" -frequency \n");
	printf("    with value\n");
	printf("    float : tone's frequency\n");
	printf(" -filename \n");
	printf("    with values\n");
	printf("    char* : sample's filenam\n");
	printf(" -loop \n");
	printf("    with values\n");
	printf("    {0,1} : play sample file looped ? (0=no,1=yes)\n");
	printf(" -amplitude \n");
	printf("    with value\n");
	printf("    float : sound's amplitude\n");
	printf(" -attenuationType \n");
	printf("    with values\n");
	printf("    0 : NONE\n");
	printf("    1 : LINEAR_FALLOFF_FACTOR\n");
	printf("    2 : LINEAR_FALLOFF\n");
	printf("    3 : INVERSE_SQUARE_FACTOR\n");
	printf("    4 : INVERSE_SQUARE_CLAMPED\n");
	printf(" -moveListener \n");
	printf("    with values\n");
	printf("    {0,1} : move listener instead of source ? (0=no,1=yes)\n");
	printf(" -soundPos \n");
	printf("    with values\n");
	printf("    float float float: X,Y,Z position of sound for initial set-up\n");
	printf(" -listenerPos \n");
	printf("    with values\n");
	printf("    float float float: X,Y,Z position of listener for initial set-up\n");
	printf(" -sleepTime \n");
	printf("    with value\n");
	printf("    int : time to sleep for tone and white noise objects\n");
	printf(" -moveFactor \n");
	printf("    with value\n");
	printf("    float : stepsize in movement of sound/listener\n");
	printf(" -maxDistance \n");
	printf("    with value\n");
	printf("    float : distance up to which the sound/listener is moved away\n");
	printf("\n");
	printf("********************************************************\n");
	}

int main(int argc,char **argv)
	{
	int moveScheme = NO_MOVESCHEME;
	int soundType = TONE;
	float frequency = 200.0;
	char *filename="error.wav";
	int loop=0;
	float amplitude = 1.0;
	int attenuationType = NONE;
	float listenerPosX = 0.0;
	float listenerPosY = 0.0;
	float listenerPosZ = 0.0;
	float soundPosX = 0.0;
	float soundPosY = 0.0;
	float soundPosZ = 0.0;
	int sleepTime=3;
	float moveFactor=0.1;
	int moveListener=0;
	float maxDistance=2.0;

	int i;
	float j;
	
	bergenServer * server = new bergenServer();
	bergenSound * sound;

	
	/* parse command line options */	
	for (i=1; i < argc; i++)
		{
		if (!strcmp(argv[i],"-moveScheme"))
			moveScheme = atoi(argv[++i]);
		else if (!strcmp(argv[i],"-soundType"))
			soundType = atoi(argv[++i]);
		else if (!strcmp(argv[i],"-frequency"))
			frequency = atof(argv[++i]);
		else if (!strcmp(argv[i],"-filename"))
			filename = argv[++i];
		else if (!strcmp(argv[i],"-loop"))
			loop = atoi(argv[++i]);
		else if (!strcmp(argv[i],"-amplitude"))
			amplitude = atof(argv[++i]);
		else if (!strcmp(argv[i],"-attenuationType"))
			attenuationType = atoi(argv[++i]);
		else if (!strcmp(argv[i],"-moveListener"))
			moveListener = atoi(argv[++i]);
		else if (!strcmp(argv[i],"-soundPos"))
			{
			soundPosX = atof(argv[++i]);			
			soundPosY = atof(argv[++i]);			
			soundPosZ = atof(argv[++i]);			
			}
		else if (!strcmp(argv[i],"-listenerPos"))
			{
			listenerPosX = atof(argv[++i]);			
			listenerPosY = atof(argv[++i]);			
			listenerPosZ = atof(argv[++i]);			
			}
		else if (!strcmp(argv[i],"-sleepTime"))
			sleepTime = atoi(argv[++i]);
		else if (!strcmp(argv[i],"-moveFactor"))
			moveFactor = atof(argv[++i]);
		else if (!strcmp(argv[i],"-maxDistance"))
			maxDistance = atof(argv[++i]);
		else if (!strcmp(argv[i],"-help"))
			{
			help();
			delete(server);
			return 0;
			}
		else
			{
			fprintf(stderr,"Warning: unknown command-line option '%s'\n",
				argv[i]);
			help();
			delete(server);
			return 0;
			}
		}
	
	/* Output Info */
	printf("********************************************************\n");
	printf(" Bergen Sound Server Spatialized Sound Extension Demo\n");
	printf(" written 2002 by Severin S. Todt - to@fh-wedel.de\n");
	printf("\n");
	/*create sound */
	printf("- - - - - - - - - - - - - - - - - - - - - - - - - - - - \n");
	printf(" Initial Phase :\n");
	if(soundType==TONE)
		{
		sound = new bergenTone(server);
		((bergenTone*)sound)->setFrequency(frequency);
		printf(" Creating tone object with frequency %.2f\n",frequency);
		}
	else if(soundType==SAMPLE)
		{
		sound = new bergenSample(filename,server);
		((bergenSample*)sound)->setLoop(loop);
		printf(" Creating sound sample object from file %s\n",filename);
		}
	else 
		{
		sound = new bergenWhiteNoise(server);
		printf(" Creating white noise object\n");
		}
	sound->setAmplitude(amplitude);
	printf(" Setting sound amplitude to %.2f\n",amplitude);

	printf(" Setting sound attenaution Type to ");
	if(attenuationType==NONE)
		{
		printf("NONE\n");
		sound->setAttenuationType(NONE);
		}
	else if(attenuationType==LINEAR_FALLOFF_FACTOR)
		{
		printf("LINEAR_FALLOFF_FACTOR\n");
		sound->setAttenuationType(LINEAR_FALLOFF_FACTOR);
		}
	else if(attenuationType==LINEAR_FALLOFF)
		{
		printf("LINEAR_FALLOFF\n");
		sound->setAttenuationType(LINEAR_FALLOFF);
		}
	else if(attenuationType==INVERSE_SQUARE_FACTOR)
		{
		printf("INVERSE_SQUARE_FACTOR\n");
		sound->setAttenuationType(INVERSE_SQUARE_FACTOR);
		}
	else if(attenuationType==INVERSE_SQUARE_CLAMPED)
		{
		printf("INVERSE_SQUARE_CLAMPED\n");
		sound->setAttenuationType(INVERSE_SQUARE_CLAMPED);
		}
	else 
		printf("DEFAULT\n");
		
	sound->setPosition(soundPosX,soundPosY,soundPosZ);
	printf(" Setting sound position to X:%.2f,Y:%.2f,Z:%.2f\n",soundPosX,soundPosY,soundPosZ);

	/* apply initial listener positioning settings*/
	server->listener()->setPosition(listenerPosX,listenerPosY,listenerPosZ);
	printf(" Setting listener position to X:%.2f,Y:%.2f,Z:%.2f\n",listenerPosX,listenerPosY,listenerPosZ);
	
	printf("- - - - - - - - - - - - - - - - - - - - - - - - - - - - \n");
	printf(" Movement Phase :\n");
	if(moveScheme==DISTANCE_RIGHT)
		{
		printf(" Movement to right");
		if(moveListener)
			{
			printf(" - Listener\n");
			sound->play();
			while(sqrt( 
				pow(((server->listener()->positionX())-(soundPosX)),2)+
				pow(((server->listener()->positionY())-(soundPosY)),2)+
				pow(((server->listener()->positionZ())-(soundPosZ)),2) ) <= maxDistance )
				{
				server->listener()->setPosition(server->listener()->positionX()+moveFactor,
				server->listener()->positionY(),server->listener()->positionZ());
				sleep(sleepTime);
				}
			}
		else	{
			printf(" - Sound\n");
			sound->play();
			while(sqrt( 
				pow(((server->listener()->positionX())-(soundPosX)),2)+
				pow(((server->listener()->positionY())-(soundPosY)),2)+
				pow(((server->listener()->positionZ())-(soundPosZ)),2) ) <= maxDistance )
				{
				soundPosX+=moveFactor;
				sound->setPosition(soundPosX,soundPosY,soundPosZ);
				sleep(sleepTime);
				}
			}
		}
	else if(moveScheme==DISTANCE_FRONT)
		{
		printf(" Movement to front");
		if(moveListener)
			{
			printf(" - Listener\n");
			sound->play();
			while(sqrt( 
				pow(((server->listener()->positionX())-(soundPosX)),2)+
				pow(((server->listener()->positionY())-(soundPosY)),2)+
				pow(((server->listener()->positionZ())-(soundPosZ)),2) ) <= maxDistance )
				{
				server->listener()->setPosition(server->listener()->positionX(),
				server->listener()->positionY(),server->listener()->positionZ()-moveFactor);
				sleep(sleepTime);
				}
			}
		else	{
			printf(" - Sound\n");
			sound->play();
			while(sqrt( 
				pow(((server->listener()->positionX())-(soundPosX)),2)+
				pow(((server->listener()->positionY())-(soundPosY)),2)+
				pow(((server->listener()->positionZ())-(soundPosZ)),2) ) <= maxDistance )
				{
				soundPosZ-=moveFactor;
				sound->setPosition(soundPosX,soundPosY,soundPosZ);
				sleep(sleepTime);
				}
			}
		}
	else if(moveScheme==DISTANCE_LEFT)
		{
		printf("Movement to left");
		if(moveListener)
			{
			printf(" - Listener\n");
			sound->play();
			while(sqrt( 
				pow(((server->listener()->positionX())-(soundPosX)),2)+
				pow(((server->listener()->positionY())-(soundPosY)),2)+
				pow(((server->listener()->positionZ())-(soundPosZ)),2) ) <= maxDistance )
				{
				server->listener()->setPosition(server->listener()->positionX()-moveFactor,
				server->listener()->positionY(),server->listener()->positionZ());
				sleep(sleepTime);
				}
			}
		else	{
			printf(" - Sound\n");
			sound->play();
			while(sqrt( 
				pow(((server->listener()->positionX())-(soundPosX)),2)+
				pow(((server->listener()->positionY())-(soundPosY)),2)+
				pow(((server->listener()->positionZ())-(soundPosZ)),2) ) <= maxDistance )
				{
				soundPosX-=moveFactor;
				sound->setPosition(soundPosX,soundPosY,soundPosZ);
				sleep(sleepTime);
				}
			}
		}
	else if(moveScheme==DISTANCE_BACK)
		{
		printf("Movement to back");
		if(moveListener)
			{
			printf(" - Listener\n");
			sound->play();
			while(sqrt( 
				pow(((server->listener()->positionX())-(soundPosX)),2)+
				pow(((server->listener()->positionY())-(soundPosY)),2)+
				pow(((server->listener()->positionZ())-(soundPosZ)),2) ) <= maxDistance )
				{
				server->listener()->setPosition(server->listener()->positionX(),
				server->listener()->positionY(),server->listener()->positionZ()+moveFactor);
				sleep(sleepTime);
				}
			}
		else	{
			printf(" - Sound\n");
			sound->play();
			while(sqrt( 
				pow(((server->listener()->positionX())-(soundPosX)),2)+
				pow(((server->listener()->positionY())-(soundPosY)),2)+
				pow(((server->listener()->positionZ())-(soundPosZ)),2) ) <= maxDistance )
				{
				soundPosZ+=moveFactor;
				sound->setPosition(soundPosX,soundPosY,soundPosZ);
				sleep(sleepTime);
				}
			}
		}
	else if(moveScheme==CIRCLE_HORIZONTAL)
		{
		printf(" Horizontal Circle");
		if(moveListener)
			{
			printf(" - Listener\n");
			j=0;
			sound->play();
			while (j<2*M_PI)
				{
				listenerPosX=sin(j)*maxDistance;
				listenerPosZ=cos(j)*maxDistance;
				server->listener()->setPosition(listenerPosX,listenerPosY,listenerPosZ);
				j+=moveFactor;
				sleep(sleepTime);
				}
			}
		else	{
			printf(" - Sound\n");
			j=0;
			sound->play();
			while (j<2*M_PI)
				{
				soundPosX=sin(j)*maxDistance;
				soundPosZ=cos(j)*maxDistance;
				sound->setPosition(soundPosX,soundPosY,soundPosZ);
				printf("PosX:%f - PosZ:%f\n",soundPosX,soundPosZ);
				j+=moveFactor;
				sleep(sleepTime);
				}
			}
		}
	else if(moveScheme==CIRCLE_VERTICAL)
		{
		printf(" Vertical Circle");
		if(moveListener)
			{
			printf(" - Listener\n");
			j=0;
			sound->play();
			while (j<2*M_PI)
				{
				listenerPosY=sin(j)*maxDistance;
				listenerPosZ=cos(j)*maxDistance;
				server->listener()->setPosition(listenerPosX,listenerPosY,listenerPosZ);
				j+=moveFactor;
				sleep(sleepTime);
				}
			}
		else	{
			printf(" - Sound\n");
			j=0;
			sound->play();
			while (j<2*M_PI)
				{
				soundPosY=sin(j)*maxDistance;
				soundPosZ=cos(j)*maxDistance;
				sound->setPosition(soundPosX,soundPosY,soundPosZ);
				//printf("PosX:%f - PosZ:%f\n",soundPosX,soundPosZ);
				j+=moveFactor;
				sleep(sleepTime);
				}
			}
		}
	else if(moveScheme==SPIRAL)
		{
		float k = 0.0;
		printf(" Spiral Movement");
		if(moveListener)
			{
			printf(" - Listener\n");
			j=0;
			sound->play();
			while(k<maxDistance)
				{
				while (j<2*M_PI)
					{
					listenerPosX=sin(j)*k;
					listenerPosZ=cos(j)*k;
					server->listener()->setPosition(listenerPosX,listenerPosY,listenerPosZ);
					j+=moveFactor;
					k+=(moveFactor/5);
					sleep(sleepTime);
					}
					j=0.0;					
				}
			}
		else	{
			printf(" - Sound\n");
			j=0;
			sound->play();
			while(k<maxDistance)
				{
				while (j<2*M_PI)
					{
					soundPosX=sin(j)*k;
					soundPosZ=cos(j)*k;
					sound->setPosition(soundPosX,soundPosY,soundPosZ);
					//printf("PosX:%f - PosZ:%f\n",soundPosX,soundPosZ);
					j+=moveFactor;
					k+=(moveFactor/5);
					sleep(sleepTime);
					}
					j=0.0;					
				}
			}
		}
	else
		{		
		printf(" No Movement, play at initial position\n");
		sound->play();
		sleep(sleepTime);
		}	
	sound->kill();
	delete(sound);
	delete(server);
	printf("********************************************************\n");
	return 0;
	}
