#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <sys/soundcard.h>
#include <stdlib.h>
#include <math.h>
#include "AudioInterface.h"

#define DEVICE_NAME "/dev/dsp"

int AudioInterface::outputRate_;
int AudioInterface::inputRate_;
ALport AudioInterface::outputPort_;
ALport AudioInterface::inputPort_;
int AudioInterface::outputChannels_;
int AudioInterface::inputChannels_;
int AudioInterface::inputBufferReady_ = 0;
int AudioInterface::inputReadError_ = 0;
int AudioInterface::bufferSize_ = 0;
short * AudioInterface::inputBuffer_ = NULL;


void AudioInterface::openOutputPort(int bufsize,int sampleRate,int channels)
	{
	outputChannels_ = channels;
	bufferSize_ = bufsize;
	outputRate_ = sampleRate;

	if (channels >= 4)
		{
		fprintf(stderr,"ERROR: 4 channel audio is not supported on Linux\n");
		outputChannels_ = 2;
		}
	if ((outputPort_ = open(DEVICE_NAME, O_WRONLY)) == -1)
	       {
	       fprintf(stderr,"Failed to open audio port for writing\n");
	       perror(DEVICE_NAME);
	       exit(-1);
	       }
	int sampleSize=16;
	if (ioctl(outputPort_, SNDCTL_DSP_SAMPLESIZE, &sampleSize) == -1)  
		{
		perror("SNDCTL_DSP_SAMPLESIZE");  
		exit(-1);
		}
	int prof=APF_NORMAL;
	ioctl(outputPort_, SNDCTL_DSP_PROFILE, &prof);
	int stereo=outputChannels_-1;
	if (ioctl(outputPort_, SNDCTL_DSP_STEREO, &stereo) == -1)  
		{
		perror("SNDCTL_DSP_STEREO");  
		exit(-1);
		}
	if (ioctl(outputPort_, SNDCTL_DSP_SPEED, &outputRate_) == -1)  
		{
		perror("SNDCTL_DSP_SPEED");  
		exit(-1);
		}
	/* Tell hardware to use 3 fragments, of size 1024 bytes (2^10) */
	int fragSize = 0x0003000A;
	if (ioctl(outputPort_, SNDCTL_DSP_SETFRAGMENT, &fragSize) == -1)  
		perror("SNDCTL_DSP_SETFRAGMENT");  
	/* Find out what the resulting fragment size actually is */
	int blockSize=0;
	if (ioctl(outputPort_, SNDCTL_DSP_GETBLKSIZE, &blockSize) != -1)
		{
		if (blockSize > 0) ;
			//snerdAudiobufferSize_ = blockSize/2;
		}
	}

void AudioInterface::openInputPort(int bufsize,int sampleRate,int channels)
	{
	inputChannels_ = channels;
	bufferSize_ = bufsize;
	inputRate_ = sampleRate;

	if ((inputPort_ = open(DEVICE_NAME, O_RDONLY)) == -1)
	       {
	       fprintf(stderr,"Failed to open audio port for writing\n");
	       perror(DEVICE_NAME);
	       exit(-1);
	       }
	int sampleSize=16;
	if (ioctl(inputPort_, SNDCTL_DSP_SAMPLESIZE, &sampleSize) == -1)  
		{
		perror("SNDCTL_DSP_SAMPLESIZE");  
		exit(-1);
		}
	int prof=APF_NORMAL;
	ioctl(inputPort_, SNDCTL_DSP_PROFILE, &prof);
	int stereo=channels-1;
	if (ioctl(inputPort_, SNDCTL_DSP_STEREO, &stereo) == -1)  
		{
		perror("SNDCTL_DSP_STEREO");  
		exit(-1);
		}
	if (ioctl(inputPort_, SNDCTL_DSP_SPEED, &inputRate_) == -1)  
		{
		perror("SNDCTL_DSP_SPEED");  
		exit(-1);
		}
	}

int AudioInterface::writeOutputPort(void *buffer,size_t bufferSize_)
	{
	return write(outputPort_, buffer, bufferSize_*2);
	}

int AudioInterface::readInputPort(void *buffer,size_t bufsize)
	{
	if (!inputBuffer_)
		inputBuffer_ = (short *) malloc(bufferSize_ * sizeof(short));
	if (inputBufferReady_)
		{
		inputReadError_ = read(inputPort_, inputBuffer_, bufferSize_*2);
		inputBufferReady_ = 0;
		}
	short* bufferPtr = (short*)buffer;
	for (int i=0; i<bufsize; i++)
		bufferPtr[i] = inputBuffer_[i*inputChannels_];
	return inputReadError_;
	}

void AudioInterface::fillInputPort(void)
	{
	inputBufferReady_ = 1;
	}
