#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <unistd.h>
#include <fcntl.h>
#include <math.h>
#include "snerdDB.h"
#include "snerdServer.h"
#include "RATSource.h"

#define RAT_EXECUTABLE "rat-4.2.23"
#define RAT_UPDATE_INTERVAL 0.2
#define MBUS_SNERD_ADDR  "(media:audio module:remote app:rat id:%lu)"
#define MBUS_ENGINE_ADDR "(media:audio module:engine app:rat session:0 id:%lu)"
#define MBUS_UI_ADDR     "(media:audio module:ui app:rat id:%lu)"

static void mbus_err_handler(int seqnum, int reason)
	{
        /* Something has gone wrong with the mbus transmission. At this time */
        /* we don't try to recover, just kill off the media engine and user  */
        /* interface processes and exit.                                     */
	
	}

static void mbus_remote_rx(char *srce, char *cmnd, char *args, void *data) {}

char RATSource::snerd_addr[60];
char RATSource::engine_addr[60];
char RATSource::ui_addr[60];
int RATSource::pid = 0;
float RATSource::updateInterval = RAT_UPDATE_INTERVAL;
#ifdef COMMON_LIBRARY
struct mbus* RATSource::m = NULL;
#else
void* RATSource::m = NULL;
#endif



// Initializes RATSource Object with initial values and calls the initial- 
// file routine either for raw file or aiff file
RATSource::RATSource(char *source, snerdDB* sDB=NULL, int listener=-1, ConfigInformation* config=NULL) :
ASound(sDB,listener,config)
	{
	killed_ = 0;
	playing_ = 0;
	sourceGain_ = 1.0 ;
	sourceAzimuth_ = 0.0;
	enabled3D_ = 0;
	lastUpdate = 0.0;
	gainThresh_ = 0.005;
	azimuthThresh_ = 1.0;
	if (source)
		{
		source_ = strdup(source);
		if (!m)
			initializeMbus();
		}
	struct timeval t;
	gettimeofday(&t,NULL);
	startTime = t.tv_sec;
	}


// Initializes MBUS communication with a running RAT process
void RATSource::initializeMbus(void)
	{
	pid = 0;
	if (getenv("BERGEN_RAT_UPDATE_INTERVAL"))
		updateInterval = atoi(getenv("BERGEN_RAT_UPDATE_INTERVAL"));
#ifdef COMMON_LIBRARY
	char ratExe[32] = {RAT_EXECUTABLE};
	if (getenv("BERGEN_RAT_EXECUTABLE"))
		strcpy(ratExe,getenv("BERGEN_RAT_EXECUTABLE"));
	char cmd[256] = {"ps -ef | grep "};
	strcat(cmd,ratExe);
	FILE* fp_ = popen(cmd,"r");
	if (!fp_)
		{
		printf("ERROR: failed to execute system command - %s\n",cmd);
		return;
		}
	char linebuf_[1024];
	while (fgets(linebuf_,sizeof(linebuf_)-1,fp_))
		{
		int currPid;
		char *tokenChar;
		tokenChar = strtok(linebuf_," ");
		int token = 1;
		while (tokenChar = strtok(NULL," "))
			{
			if (token == 1)
				currPid = atoi(tokenChar);
			if (token == 7 && strlen(tokenChar) >= strlen(ratExe))
				{
				char truncToken[128];
				strcpy(truncToken,tokenChar+(strlen(tokenChar)-strlen(ratExe)));
				if (!strcmp(truncToken,ratExe))
					pid = currPid;
				}
			token++;
			}		
		}
	pclose(fp_);
	if (!pid)
		{
		printf("WARNING: RAT process not found\n");
		return;
		}
	sprintf(engine_addr,MBUS_ENGINE_ADDR, pid);
	sprintf(ui_addr,MBUS_UI_ADDR, pid);
	sprintf(snerd_addr,MBUS_SNERD_ADDR, pid);
	m = mbus_init(mbus_remote_rx, mbus_err_handler, snerd_addr);
	if (m == NULL)
		printf("ERROR: Could not initialize Mbus\n");
#endif
	}


// RATSource destructor
RATSource::~RATSource(void)
	{
	}


//  Sets RAT source identifier
void RATSource::setSource(char* source)
	{
	if (source)
		{
		source_ = strdup(source);
		if (!m)
			initializeMbus();
		}
	}


// Sends mbus messages to set a source gain
void RATSource::setSourceGain(float val)
	{
	sourceGain_ = val;
	if (m)
		{
#ifdef COMMON_LIBRARY
		mbus_qmsgf(m, engine_addr, 1, "rtp.source.gain", "\"%08s\" %.2f", source_, sourceGain_);
		mbus_qmsgf(m, ui_addr, 1, "rtp.source.gain", "\"%08s\" %.2f", source_, sourceGain_);
#endif
#ifdef DEBUG
		printf("message: rtp.source.gain with: \"%08s\" %.2f\n",source_,sourceGain_);
#endif
		}
	}


// Sends mbus messages to enable RAT 3D rendering
// This functionality is not implemented on SGI platform
void RATSource::set3DEnable(int val)
	{
	enabled3D_ = val;
	if (m)
		{
#ifdef COMMON_LIBRARY
#ifndef __sgi
		mbus_qmsgf(m, engine_addr, 1, "audio.3d.enabled", "%d", enabled3D_);
		mbus_qmsgf(m, ui_addr, 1, "audio.3d.enabled", "%d", enabled3D_);
#endif
#endif
#ifdef DEBUG
		printf("message: audio.3d.enabled with: %d\n",enabled3D_);
#endif
		}
	}


// Sends mbus messages to set source azimuth for 3D positional sound
void RATSource::setSource3DAzimuth(float val)
	{
	sourceAzimuth_ = val;
	if (m)
		{
#ifdef COMMON_LIBRARY
		mbus_qmsgf(m, engine_addr, 1, "audio.3d.user.settings", "\"%08s\" \"HRTF\" 32 %d",source_,(int)sourceAzimuth_);
		mbus_qmsgf(m, ui_addr, 1, "audio.3d.user.settings", "\"%08s\" \"HRTF\" 32 %d",source_,(int)sourceAzimuth_);
#endif
#ifdef DEBUG
		printf("message: audio.3d.user.settings with \"%08s\" HRTF 32 %d\n",source_,(int)sourceAzimuth_);
#endif
		}
	}


// Returns the source identifier
char* RATSource::source(void)
	{
	return source_;
	}


// Sets the playing_ flag for the calculation of gain and azimuth
void RATSource::play(void)
	{
	playing_ = 1;
	}


// Sets back the playing_ flag 
void RATSource::stop(void)
	{
	playing_ = 0;
	}


// Unsets the playing_ flag
void RATSource::pause(void)
	{
	playing_ = 0;
	}


// Sets the killed_ flag and closes the associated audio file
void RATSource::kill(void)
	{
	killed_ = 1;
	}


// Calculates the current azimuth for a source
float RATSource::sourceAzimuth(void)
	{
	Listener * listener;
	float sourceX,sourceY,sourceZ,sourceLength;
	double azimuth = 0.0;
	
	listener = (Listener*)db_->findListener(listenerHandle_);
	if(listener)
		{	
		//track listener movement
		sourceX=positionX_-listener->positionX();
		sourceY=positionY_-listener->positionY();
		sourceZ=positionZ_-listener->positionZ();
		
		//calculate unit vector
		sourceLength=(float)sqrt(pow(sourceX,2)+pow(sourceY,2)+pow(sourceZ,2));
		
		//if source position is not equal to listener position
		if(sourceLength)
			{
			//build unit source vector
			sourceX/=sourceLength;
			sourceY/=sourceLength;
			sourceZ/=sourceLength;	
			azimuth = (-asin((double)(sourceX))/M_PI)*180.0;
			}
		}
	return azimuth;
	}


// Updates the mbus timeout and calculates source gain and azimuth
short * RATSource::updateBuffer(void)
	{
	updatePosition();
	struct timeval t;
	gettimeofday(&t,NULL);
	float currTime;
	currTime = (t.tv_sec-startTime) + t.tv_usec*(1.0f/1000000.0f);
	if ((currTime - lastUpdate) > updateInterval)
		{
		if (m)
			{
#ifdef COMMON_LIBRARY
			mbus_send(m);
			mbus_heartbeat(m, 1);
			mbus_retransmit(m);
			struct timeval timeout;
			timeout.tv_sec  = 0;
			timeout.tv_usec = 20000;
			mbus_recv(m, NULL, &timeout);
#endif
			}

		if (playing_)
			{
			float gain = attenuatedGain();
			if (fabs(gain-sourceGain_) > gainThresh_)
				setSourceGain(gain);
			if (enabled3D_)
				{
				switch (config_->mode)
					{
					case AMBISONIC:
					case STEREO:
						float azimuth;
						azimuth = sourceAzimuth();
						if (fabs(azimuth-sourceAzimuth_) > azimuthThresh_)
							setSource3DAzimuth(azimuth);
						break;
					default:
						break;
					}
				}
			}
		lastUpdate = currTime;
		}
	return NULL;
	}


// A RAT source file can be removed if it was killed before 
int RATSource::removable(void)
	{
	return killed_;
	}


// Returns the value of the playing flag to determine if RATSource object is playing
int RATSource::active(void)
	{
	return 1;
	}


// Executes the right action refereing to the messages coming in
void RATSource::message(char **msg,int numStrings)
	{
	if (!strcasecmp(msg[0],"source"))
		{
		if (numStrings > 1)
			setSource(msg[1]);
		}
	else if (!strcasecmp(msg[0],"gain"))
		{
		if (numStrings > 1)
			setSourceGain(atof(msg[1]));
		}
	else if (!strcasecmp(msg[0],"enable"))
		{
		if (numStrings > 1)
			set3DEnable(atoi(msg[1]));
		}
	else if (!strcasecmp(msg[0],"azimuth"))
		{
		if (numStrings > 1)
			setSource3DAzimuth(atof(msg[1]));
		}
	else
		ASound::message(msg,numStrings);
	}
