#include <malloc.h>
#include <math.h>
#include <stdlib.h>
#include "WhiteNoise.h"


// Initialises White Noise Object with default values
WhiteNoise::WhiteNoise(snerdDB* sDB=NULL, int listener=-1, ConfigInformation* config=NULL) :
ASound(sDB,listener,config)
	{
	playing_ = 0;
	killed_ = 0;
	}


// Stops audio play and set killed flag by calling kill 
WhiteNoise::~WhiteNoise(void)
	{
	kill();
	}


// Returns a list of samples to be played next 
short * WhiteNoise::updateBuffer(void)
	{
	updatePosition();
	if (playing_)
		{
		int i,k;
		k=0;
		switch (config_->mode)
			{
			case STEREO:
				for (i=0; (i < (bufferSize_/2)); i++)
					{
					float value = (random() % 65536 - 32768) * attenuatedGain();
					buffer_[k++] = (short) (value * spatializedGain(0));
					buffer_[k++] = (short) (value * spatializedGain(1));		
					}
				break;
			case AMBISONIC:	
				for (i=0; (i < (bufferSize_/4)); i++)
					{
					float value = (random() % 65536 - 32768) * attenuatedGain();
					buffer_[k++] = (short) (value * spatializedGain(0));
					buffer_[k++] = (short) (value * spatializedGain(1));		
					buffer_[k++] = (short) (value * spatializedGain(2));
					buffer_[k++] = (short) (value * spatializedGain(3));		
					}
				break;
			default:
				for (i=0; i < bufferSize_; i++)
					buffer_[i] = (short) ((random() % 65536 - 32768) * attenuatedGain());
				break;
			}
		}
	return buffer_;
	}


// Returns value of killed_ flag to determine if WhiteNoise Object is allowed 
// to be killed.
int WhiteNoise::removable(void)
	{
	return killed_;
	}


// Returns the value of the playing flag to determine if WhiteNoise object is playing
int WhiteNoise::active(void)
	{
	return playing_;
	}


// Sets the playing_ flag to determine that this White Noise Object is 
// to be played.
void WhiteNoise::play(void)
	{
	playing_ = 1;
	}


// Unsets the playing_ flag to determine that this White Noise Object is
// not to be played any more.
void WhiteNoise::stop(void)
	{
	playing_ = 0;
	}


// Unsets the playing_ flag to determine that this White Noise Object is 
// not to be played any more.
void WhiteNoise::pause(void)
	{
	playing_ = 0;
	}


// Stops replay of the WhiteNoise Object to ensure that server is not 
// referncing it any more in play loop and sets the killed_ flag 
void WhiteNoise::kill(void)
	{
	stop();
	killed_ = 1;
	}
